"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = require("express");
const core_types_1 = require("./types/core.types");
const helpers_1 = require("./helpers");
/**
 * Recursive. Takes the ZemiRoutes passed in and returns an Express router
 * (either created or specified as the second argument) with each ZemiRoute as
 * an express route under it.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [router] {Router} - An Express router to be used for adding ZemiRoutes; router is auto-created if not passed in.
 * @param [__routeDefinitions] {Record<string, ZemiRouteDefinition>} - The route definitions for this zemi function. Initialized if not passed-in.
 * @param [__parentPath] {string} - Path of parent, use to identify current path definition.
 * @returns {Router} - A router with all ZemiRoutes built into it.
 * @type{(routes: Array<ZemiRoute>, router: Router)=> Router}
 */
function Zemi(routes, router = (0, express_1.Router)({ mergeParams: true }), __routeDefinitions = (0, helpers_1.buildRouteDefinitions)(routes), __parentPath) {
    const allowedResponseHttpCodes = (0, helpers_1.buildResponsesPerNamedRoute)(routes);
    router.use(function (request, response, next) {
        request.routeDefinitions = __routeDefinitions;
        request.allowedResponseHttpCodes = allowedResponseHttpCodes;
        next();
    });
    routes.forEach((route) => {
        const path = (0, helpers_1.paramPathToValidExpressPath)(route.path);
        const fullPath = __parentPath ? `${__parentPath}${path}` : path;
        route.middleware &&
            route.middleware.forEach((middleware) => router.use((request, response, next) => middleware(request, response, next)));
        const methods = Object.values(core_types_1.ZemiMethod);
        methods.forEach((method) => {
            const handlerDefinition = route[method];
            if (handlerDefinition && handlerDefinition.handler) {
                // const routeDef: ZemiRouteDefinition = buildRouteDefinition(
                //   path,
                //   route.name
                // );
                const routeDef = Object.keys(__routeDefinitions)
                    .map((rdk) => __routeDefinitions[rdk])
                    .filter((rd) => rd.path === fullPath)[0];
                router[method](path, (request, response, next) => handlerDefinition.handler(request, response, next, routeDef));
            }
        });
        route.routes &&
            router.use(path, Zemi(route.routes, router, __routeDefinitions, fullPath));
    });
    return router;
}
exports.default = Zemi;
//# sourceMappingURL=core.js.map