"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.paramPathToOpenApiParamObject = exports.paramPathToOpenApiPath = exports.paramPathToValidExpressPath = exports.buildResponsesPerNamedRoute = exports.buildRouteDefinitions = exports.buildRouteDefinition = void 0;
const core_types_1 = require("./types/core.types");
/**
 * Builds a route-definition with the name and path specified.
 * Will build an array of strings with the parameters specified in the path, and
 * a reverse function -- which will substitute params in the path for values
 * passed in via an object -- to the route-definition.
 * @param path {string} - The path for this route-definition.
 * @param name {string} - The name for this route-definition.
 * @return {ZemiRouteDefinition} - A route-definition with the name, path, parameters in the path, and reverse function for said path.
 * @type{(path: string, name:string)=> ZemiRouteDefinition}
 */
function buildRouteDefinition(path, name) {
    const pathArray = path.split("/");
    const parameters = pathArray
        .filter((p) => p.includes(":"))
        .map((p) => p.split(":")[1]);
    const reverse = (paramValues) => {
        const pathParts = pathArray.map((p) => {
            if (p.includes(":")) {
                const key = p.split(":")[1];
                return paramValues[key];
            }
            else
                return p;
        });
        return pathParts.join("/");
    };
    return { name, path, parameters, reverse };
}
exports.buildRouteDefinition = buildRouteDefinition;
/**
 * Recursive. Builds a route-definition for each ZemiRoute in the array.
 * Converts zemi paths (path that has params with their types built in)
 * into valid Express paths. Used to add route-definitions to ZemiRequest
 * object.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [prefix] {NamedRoute} - The prefix to use for this route, if nested, so that its parent's path is added to its path.
 * @returns {Record<string, ZemiRouteDefinition>} - An object where each key is a route name and the value is that routes' route-definition.
 * @type{(routes: Array<ZemiRoute>, prefix?: NamedRoute)=> Record<string, ZemiRouteDefinition>}
 */
function buildRouteDefinitions(routes, prefix) {
    const namedRoutes = routes.map((r) => {
        if (r.name) {
            const name = prefix ? `${prefix.name}-${r.name}` : r.name;
            const dirtyPath = prefix ? `${prefix.path}${r.path}` : r.path;
            const path = paramPathToValidExpressPath(dirtyPath);
            const mine = {
                [name]: buildRouteDefinition(path, name),
            };
            if (r.routes) {
                const toReturn = Object.assign({}, buildRouteDefinitions(r.routes, { name, path }), mine);
                return toReturn;
            }
            else {
                return mine;
            }
        }
    });
    return Object.assign({}, ...namedRoutes);
}
exports.buildRouteDefinitions = buildRouteDefinitions;
/**
 * Recursive. Builds an object with route names as keys and values are objects mapping
 * HTTP methods to array of response codes for those same methods. Used to add
 * allowedResponseCodes to ZemiRequest object.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [prefix] {NamedRoute} - The prefix to use for this route, if nested, so that its parent's path is added to its path.
 * @returns {Record<string, Record<string, Array<string>>>} - An object where each key is a route name and the value is an object whose keys map to an array of allowed response codes, as strings.
 * @type{(routes: Array<ZemiRoute>, prefix?: NamedRoute)=> Record<string, Record<string, Array<string>>>}
 */
function buildResponsesPerNamedRoute(routes, prefix) {
    const namedRoutes = routes.map((r) => {
        if (r.name) {
            const name = prefix ? `${prefix}-${r.name}` : r.name;
            const methods = Object.values(core_types_1.ZemiMethod);
            const responsesPerMethod = methods.map((method) => {
                const hd = r[method];
                if (hd && hd.responses) {
                    return { [method]: Object.keys(hd.responses) };
                }
            });
            const mine = { [name]: Object.assign({}, ...responsesPerMethod) };
            if (r.routes) {
                return Object.assign({}, buildResponsesPerNamedRoute(r.routes, name), mine);
            }
            else {
                return mine;
            }
        }
    });
    return Object.assign({}, ...namedRoutes);
}
exports.buildResponsesPerNamedRoute = buildResponsesPerNamedRoute;
/**
 * Converts a zemi-style path -- where parameters are specified as
 * `{param|type}` -- to a valid Express path (parameters are defined as
 * `:param`). Used to generate paths for Express routes, from ZemiRoutes.
 * @param path {string} - A path that has parameters specified as `{param|type}`.
 * @return {string} - A valid Express path, that has parameters specified as `:param`.
 * @type{(path: string)=> string}
 */
function paramPathToValidExpressPath(path) {
    const pathBits = path.split("/").map((p) => {
        if (p[0] === "{" && p[p.length - 1] === "}") {
            const paramKey = p.split("{")[1].split("|")[0];
            return `:${paramKey}`;
        }
        else {
            return p;
        }
    });
    return pathBits.join("/");
}
exports.paramPathToValidExpressPath = paramPathToValidExpressPath;
/**
 * Converts a zemi-style path -- where parameters are specified as
 * `{param|type}` -- to a valid OpenApi path (parameters are defined as
 * `{param}`). Used to generate paths for OpenApi spec, from ZemiRoutes.
 * @param path {string} - A path that has parameters specified as `{param|type}`.
 * @return {string} - A valid OpenApi path, that has parameters specified as `{param}`.
 * @type{(path: string)=> string}
 */
function paramPathToOpenApiPath(path) {
    const pathBits = path.split("/").map((p) => {
        if (p[0] === "{" && p[p.length - 1] === "}") {
            const paramKey = p.split("{")[1].split("|")[0];
            return `{${paramKey}}`;
        }
        else {
            return p;
        }
    });
    return pathBits.join("/");
}
exports.paramPathToOpenApiPath = paramPathToOpenApiPath;
/**
 * Converts a zemi-style path -- where parameters are specified as
 * `{param|type}` -- to an OpenApi ParameterObject.
 * Used when generating ParameterObjects, from ZemiRoutes, for OpenApi spec generation.
 * @param path {string} - A path that has parameters specified as `{param|type}`.
 * @return {Array<OpenApiParameterObject>} - An array of OpenApi ParameterObject.
 * @type{(path: string)=> Array<OpenApiParameterObject>}
 */
function paramPathToOpenApiParamObject(path) {
    const paramBits = path
        .split("/")
        .filter((p) => p[0] === "{" && p[p.length - 1] === "}");
    const noBracks = paramBits.map((pb) => pb.substring(1, pb.length - 1));
    const paramsSeparatedList = noBracks.map((p) => p.split("|"));
    return paramsSeparatedList.map(([name, type]) => {
        return {
            name,
            in: "path",
            schema: { type },
            required: true,
        };
    });
}
exports.paramPathToOpenApiParamObject = paramPathToOpenApiParamObject;
//# sourceMappingURL=helpers.js.map