"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.asyncWriteFile = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const core_types_1 = require("./types/core.types");
const helpers_1 = require("./helpers");
/**
 * Generates OpenApi Operation Objects from ZemiRoutes, path-parameters, and
 * definition-parameters passed in. This method folds all parameters at a path
 * level, method level, and parameters generated from the zemi-style path
 * specified in the ZemiRoute, into a single collection. Used to automatically
 * generate each ZemiRoute's method spec for final OpenApi doc.
 * @param route {ZemiRoute} - An ZemiRoute.
 * @param pathParams {Array<OpenApiParameterObject>} - An array of OpenApi Operation objects for a path (for all methods for said path), specified in the ZemiRoute.
 * @param definitionParams {Array<OpenApiParameterObject>} - An array of OpenApi Operation objects for a method, specified in the ZemiRoute.
 * @return {Array<OpenApiOperationObject>} - An array of OpenApi Operation objects generated from the methods of the specified ZemiRoute, the {pathParams} argument, and {definitionParams} argument.
 * @type{(  routes: Array<ZemiRoute>, parentPath: string)=> Array<OpenApiOperationObject>}
 */
function buildMethodDocs(route, pathParams, definitionParams) {
    return Object.values(core_types_1.ZemiMethod).map((method) => {
        const methodDoc = {};
        const hd = route[method];
        if (hd) {
            const methodParams = hd.parameters || [];
            const parameters = [...pathParams, ...definitionParams, ...methodParams];
            const { description, summary, responses, operationId, requestBody, tags, security, } = hd;
            methodDoc[method] = {
                parameters,
                description,
                summary,
                responses,
                operationId,
                requestBody,
                tags,
                security,
            };
            return methodDoc;
        }
    });
}
/**
 * Generates OpenApi PathItem Objects from ZemiRoutes passed in.
 * Used to automatically generate each ZemiRoute's path spec for final OpenApi
 * doc.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [parentPath] {string} - The parent's path to use for this route, if nested, so that it is added to this path.
 * @return {Array<OpenApiPathItemObject>} - An array of OpenApi PathItemObject generated from specified ZemiRoutes.
 * @type{(  routes: Array<ZemiRoute>, parentPath: string)=> Array<OpenApiPathItemObject>}
 */
function buildPathDocs(routes, parentPath) {
    return routes.flatMap((route) => {
        const path = route.path;
        const paramPath = parentPath ? `${parentPath}${path}` : path;
        const pathParams = (0, helpers_1.paramPathToOpenApiParamObject)(paramPath);
        const definitionParams = route.parameters || [];
        const methods = buildMethodDocs(route, pathParams, definitionParams);
        const mine = [
            { [(0, helpers_1.paramPathToOpenApiPath)(paramPath)]: Object.assign({}, ...methods) },
        ];
        if (route.routes) {
            const childRoutesList = buildPathDocs(route.routes, path);
            return [...mine, ...childRoutesList];
        }
        else {
            return mine;
        }
    });
}
/**
 * Asynchronously write data to a file.
 * @param writeFile {(path: string, data: any, options: object) => void} - A function that will write data to a file; really just a way to pass in fs.promises.writeFile.
 * @param path {string} - The path to write to.
 * @param data {any} - Any data that we want to write to a file.
 * @return {Promise<void>}
 * @type {(
 *   writeFile: (path: string, data: any, options: object) => void,
 *   path: string,
 *   data: any
 * )=> Promise<void>}
 */
function asyncWriteFile(writeFile, path, data) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield writeFile(path, data, { flag: "w" });
        }
        catch (err) {
            console.log(`-- Something went wrong when trying to write ${path}:`);
            console.log(err);
        }
    });
}
exports.asyncWriteFile = asyncWriteFile;
/**
 * Generate an OpenApi JSON file that has the passed-in spec and ZemiRoutes
 * defined in it. If correctly specified, this should generate an OpenAPI
 * compliant spec. This method should write to the current dir, but can be
 * overridden via {options}; it will also return and log the generated JSON.
 *
 * Note that this is **not** an OpenApi validator.
 * @param doc {OpenApiDoc} - An OpenApi spec as an {OpenApiDoc} to be used when generating the OpenApi JSON.
 * @param routes {Array<ZemiRoute>} - An array of ZemiRoutes.
 * @param [options] {ZemiOpenApiSpecGenerationOptions} - Options to use when generating the OpenApi doc.
 * @return {OpenApiDoc} - The OpenApi JSON generated from the passed in OpenApi doc and ZemiRoutes.
 * @type{(doc: OpenApiDoc, routes:Array<ZemiRoute>, options: ZemiOpenApiSpecGenerationOptions)=> Promise<OpenApiDoc>}
 */
function ZemiOpenApiSpecGenerator({ doc, routes, options, }) {
    return __awaiter(this, void 0, void 0, function* () {
        const whereToWrite = options && options.path
            ? options.path
            : (0, path_1.join)((0, path_1.dirname)(require.main.filename), "openapi.json");
        const pathDocs = buildPathDocs(routes);
        const paths = Object.assign({}, ...pathDocs);
        const document = Object.assign({}, doc, { paths });
        console.log("\n====================");
        console.log("OpenAPI spec:");
        console.log("====================\n");
        console.log(JSON.stringify(document));
        console.log("\n--\n");
        yield asyncWriteFile(fs_1.promises.writeFile, whereToWrite, JSON.stringify(document)).then(() => console.log(`Finished writing OpenAPI spec to ${whereToWrite}.\n`));
        return document;
    });
}
exports.default = ZemiOpenApiSpecGenerator;
//# sourceMappingURL=openapi.js.map